;==========================================================
; XREFBIND.LSP 
; Copyright 1995 by Looking Glass Microproducts
;==========================================================
; Replace existing Xrefs with Insertions
; 
;==========================================================
(defun C:XREFBIND (/ ERROR OLD_ERROR NOTRANS ITEM BITSET 
                   GET_BLOCKS SET_PATTERN SET_CHOICES 
                   SET_ALL SET_NONE GET_XREFS BIND 
                   XREFBIND_MAIN)

   ;=======================================================
   ; Error Handler
   ;=======================================================
   (defun ERROR (S)
      (if (not
             (member
                S
                '("Function cancelled" "console break")
             )
          )
         (alert S)
      )
      (while (/= 0 (getvar "cmdactive")) (command))
      (command
         "_.undo" "_end"
      )
      (princ "\nUndoing...")
      (command "_.undo" 1)
      (setq *error* OLD_ERROR)
      (princ)
   )
   ;=======================================================
   ; Disallow transparent invocation of routine.
   ;=======================================================
   (defun NOTRANS ()
      (cond
         ((zerop (getvar "cmdactive")))
         ((alert
         "\nThis command may not be invoked transparently."
          )
         )
      )
   )
   ;=======================================================
   ; Return an item n from an association list.
   ;=======================================================
   (defun ITEM (N L) (cdr (assoc N L)))

   ;=======================================================
   ; Return T if any bit of A matches B 
   ;=======================================================
   (defun BITSET (A B) (/= 0 (logand A B)))

   ;=======================================================
   ; Get names of blocks in drawing
   ;=======================================================
   (defun GET_BLOCKS (/ REWIND BNAME FLAGS PATH OVERLAY 
                     ENTRY BLOCKS XREFS)
      (setq REWIND t)
      (while (setq ENTRY (tblnext "block" REWIND))
         (setq
            REWIND  nil
            BNAME   (strcase (ITEM 2 ENTRY))
            PATH    (ITEM 1 ENTRY)
            FLAGS   (ITEM 70 ENTRY)
            OVERLAY (BITSET FLAGS 8)
         )
         (cond
            ;--------------------------------------
            ; ignore anonymous or dependent blocks
            ;--------------------------------------
            ((BITSET FLAGS (+ 1 16)))

            ;--------------------------------------
            ; note standard blocks
            ;--------------------------------------
            ((not (BITSET FLAGS 4))
               (setq
                  BLOCKS (cons BNAME BLOCKS)
               )
            )

            ;--------------------------------------
            ; note xrefs
            ;--------------------------------------
            (t
               ;-----------------------------------
               ; Add extension to path if necessary
               ;-----------------------------------
               (setq PATH (strcase PATH t))
               (if (wcmatch PATH "~*`.*")
                  (setq
                     PATH (strcat PATH ".dwg")
                  )
               )
               (setq
                  XREFS (cons 
                           (list BNAME OVERLAY PATH) 
                           XREFS
                        )
               )
            )
         )
      )
      (list BLOCKS XREFS)
   )
   ;=======================================================
   ; Display the Xrefs that match the pattern
   ;=======================================================
   (defun SET_PATTERN (VALUE / XREF N NEW_CHOICES)
      (if (/= VALUE PATTERN)
         (progn
            (setq
               PATTERN     (strcase VALUE)
               CHOICE_LIST nil
               N           0
               NEW_CHOICES ""
            )
            ;----------------------------------------------
            ; Note the xrefs that match the pattern
            ;----------------------------------------------
            (foreach XREF XREF_NAMES
               (if (wcmatch XREF PATTERN)
                  (progn
                     ;-------------------------------------
                     ; If it was selected before, 
                     ; select it again.
                     ;-------------------------------------
                     (if (member XREF CHOICES)
                        (setq
                           NEW_CHOICES (strcat
                                          NEW_CHOICES
                                          " "
                                          (itoa N)
                                       )
                        )
                     )
                     ;-------------------------------------
                     ; Add this xref to the choice_list
                     ;-------------------------------------
                     (setq
                        CHOICE_LIST (cons XREF CHOICE_LIST)
                        N           (1+ N)
                     )
                  )
               )
            )
            (setq CHOICE_LIST (reverse CHOICE_LIST))

            ;----------------------------------------------
            ; Display the list of matching xrefs
            ;----------------------------------------------
            (start_list "choices")
            (mapcar
               '(lambda
                  (XREF / TEMP PATH XTYPE)
                  (setq
                     TEMP  (ITEM XREF XREFS)
                     XTYPE (if (car TEMP) 
                               "Overlay" 
                               "Attach"
                           )
                     PATH  (cadr TEMP)
                  )
                  (add_list
                     (strcat XREF "\t" PATH "\t" XTYPE)
                  )
               )
               CHOICE_LIST
            )
            (end_list)

            ;----------------------------------------------
            ; Select the selected xrefs
            ;----------------------------------------------
            (setq NEW_CHOICES (substr NEW_CHOICES 2))
            (set_tile
               "choices"
               NEW_CHOICES
            )
            ;----------------------------------------------
            ; And note their selection
            ;----------------------------------------------
            (SET_CHOICES NEW_CHOICES)
         )
      )
   )
   ;=======================================================
   ; Set Choices
   ;=======================================================
   (defun SET_CHOICES (VALUE)
      (setq
         CHOICES (mapcar
                    '(lambda (X) (nth X CHOICE_LIST))
                    (read
                       (strcat "(" VALUE ")")
                    )
                 )
      )
   )
   ;=======================================================
   ; Select All the displayed Xrefs
   ;=======================================================
   (defun SET_ALL (/ N NEW_CHOICES)
      (setq N (length CHOICE_LIST) NEW_CHOICES "")
      (while (> N 0)
         (setq
            N           (1- N)
            NEW_CHOICES (strcat " " (itoa N) NEW_CHOICES)
         )
      )
      (setq NEW_CHOICES (substr NEW_CHOICES 2))
      (set_tile
         "choices"
         NEW_CHOICES
      )
      (SET_CHOICES NEW_CHOICES)
   )
   ;=======================================================
   ; Select None of the displayed Xrefs
   ;=======================================================
   (defun SET_NONE (/ N TEMP)
      (set_tile "choices" "")
      (SET_CHOICES "")
   )
   ;=======================================================
   ; Get name of existing Xrefs with a dialog box
   ;=======================================================
   (defun GET_XREFS (XREFS / PATTERN CHOICE_LIST CHOICES 
                    DCL_ID XREF_NAMES)
      (cond
         ((minusp 
              (setq 
                  DCL_ID (load_dialog "xrefbind.dcl")
               )
          )
            (alert
               "XREFBIND.DCL -- Can't load file."
            )
         )
         ((not (new_dialog "xrefbind" DCL_ID)))
         (t
            (setq XREF_NAMES (mapcar 'car XREFS))
            (if ACAD_STRLSORT
               (setq
                  XREF_NAMES (ACAD_STRLSORT XREF_NAMES)
               )
            )
            (set_tile "pattern" "*")
            (SET_PATTERN "*")
            (action_tile
               "choices"
               "(set_choices $value)"
            )
            (action_tile "pattern" "(set_pattern $value)")
            (action_tile
               "all"
               "(set_all)"
            )
            (action_tile "none" "(set_none)")
            (if (zerop (start_dialog))
               (setq CHOICES nil)
            )
         )
      )
      CHOICES
   )
   ;=======================================================
   ; Bind a single Xref
   ;=======================================================
   (defun BIND (XREF PATH / SS INSERTS)
      ;---------------------------------------------
      ; Find the insertions of the old xrefs
      ;---------------------------------------------
      (setq
         SS (ssget 
               "x" (list (cons 0 "INSERT") (cons 2 XREF))
            )
         N  (if SS (sslength SS) 0)
      )
      ;---------------------------------------------
      ; Replace them with insertions of the new xref
      ;---------------------------------------------
      (if SS
         (princ
            (strcat
               "\nBinding "
               (itoa N)
               " insertion"
               (if (= 1 N) "" "s")
               " of "
               XREF
               "..."
            )
         )
      )
      (while (> N 0)
         (setq
            N       (1- N)
            INSERTS (cons (entget (ssname SS N)) INSERTS)
         )
      )
      (command "_.xref" "_detach" XREF)
      (command
         "_.insert"
         (strcat XREF "=" PATH)
      )
      (command)
      (mapcar 'entmake INSERTS)
   )
   ;=======================================================
   ; XrefBind Main
   ;=======================================================
   (defun XREFBIND_MAIN (/ TEMP BLOCKS XREFS CHOICES XREF 
                        PATH) 
      (cond
         ;-------------------------------------------------
         ; Get the names of the xrefs and blocks in drawing
         ;-------------------------------------------------
         ((progn
             (setq
                TEMP   (GET_BLOCKS)
                BLOCKS (car TEMP)
                XREFS  (cadr TEMP)
             )
             (null XREFS)
          )
            (alert "This drawing contains no Xrefs.")
         )
         ;-------------------------------------------------
         ; Get the name of the xrefs to replace
         ;-------------------------------------------------
         ((null (setq CHOICES (GET_XREFS XREFS))))
         (t
            (foreach XREF CHOICES
               (setq PATH (caddr (assoc XREF XREFS)))
               (BIND
                  XREF
                  PATH
               )
            )
         )
      )
   )
   ;=======================================================
   ; Start of C:XREFBIND
   ;=======================================================
   (if (NOTRANS)
      (progn
         (setq OLD_ERROR *error* *error* ERROR)
         (setvar
            "cmdecho" 0
         )
         (command "_.undo" "_group")
         (XREFBIND_MAIN)
         (command "_.undo" "_end")
         (setq *error* OLD_ERROR)
      )
   )
   (princ)
)
(princ
   (strcat
      "\nXREFBIND.LSP -- (c) 1995 "
      "by Looking Glass Microproducts"
   )
)
(princ)

